#include <stdio.h>
#include <stdlib.h>
#include <cairo.h>
#include <cairo-pdf.h>


int read_file(const char *filename, unsigned char **data, unsigned int *len)
{
    FILE *f = fopen(filename, "rb");
    if (f == NULL)
    {
        *data = NULL;
        return -1; // -1 means file opening fail
    }
    fseek(f, 0, SEEK_END);
    *len = ftell(f);
    fseek(f, 0, SEEK_SET);
    *data = malloc(*len);
    if (1 != fread(*data, *len, 1, f))
    {
        free(*data);
        return -2; // -2 means file reading fail
    }
    fclose(f);
    return 1;
}


int main(void)
{
    cairo_surface_t *image_surface;
    cairo_surface_t *surface;
    cairo_t *cr, *ir;
    
    surface = cairo_pdf_surface_create("testmime.pdf", 504, 648);
    cr = cairo_create(surface);
    
    image_surface = cairo_surface_create_similar(surface,
                    CAIRO_CONTENT_COLOR_ALPHA, 504, 648);

    cairo_set_source_rgb(cr, 0, 0, 0);
    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_NORMAL,
        CAIRO_FONT_WEIGHT_NORMAL);
    cairo_set_font_size (cr, 12.0);

    cairo_move_to(cr, 10.0, 50.0);
    cairo_show_text(cr, "Test cairo set-mime type in PDF.");

    int w, h;
    unsigned char *data;
    unsigned int len;
    int read_status;
    read_status = read_file("testmime.jpg", &data, &len);
    if (read_status != 1){
        return;
    }
    
    cairo_surface_t *image;

    if(1){
    /* FIXME This do not create a correct image since the
     * data are binary JPG and not pixel based ARGB32 data
     */
    image = cairo_image_surface_create_for_data (data,
                 CAIRO_FORMAT_ARGB32, 128, 128,
                 cairo_format_stride_for_width(
                     CAIRO_FORMAT_ARGB32, 128));
    } else {
    /* FIXME why is it not possible to use this to create an empty
     * image and then apply the mime data ?
     * When using this function, no image is drawn.
     */
    image = cairo_image_surface_create( CAIRO_FORMAT_ARGB32, 128, 128);
    }
    
    cairo_surface_set_mime_data (image, CAIRO_MIME_TYPE_JPEG,
                 data, len,
                 free, data);
    
    w = cairo_image_surface_get_width (image);
    h = cairo_image_surface_get_height (image);
    
    cairo_translate (cr, 128.0, 128.0);
    cairo_rotate (cr, 3.1415/4);
    cairo_scale  (cr, 256.0/w, 256.0/h);
    cairo_translate (cr, -0.5*w, -0.5*h);

    cairo_set_source_surface (cr, image, 0, 0);
    
    cairo_new_path(cr);
    cairo_rectangle(cr, 0, 0, 150, 150);
    cairo_clip(cr);

//  cairo_paint (cr);
    cairo_paint_with_alpha(cr, 0.5);

    cairo_surface_destroy(image);

    cairo_show_page(cr);
    cairo_surface_finish (surface);
    cairo_surface_destroy(surface);
    cairo_destroy(cr);

    return 0;
}